/* ----------------------------------------------------------------------------
 * Raima Database Manager
 *
 * Copyright (c) 2010 Raima Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Raima LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 * ----------------------------------------------------------------------------
 */

/** \file rdmdateapi.h
 *  \brief Header for the public Date API
 */

#include "rdmdatetimetypes.h"
#include "rdmretcodetypes.h"

#if defined(RDM_UNICODE)
#define rdm_dateDayAbr RDM_T_FUNC (rdm_dateDayAbr)
#define rdm_dateDayName RDM_T_FUNC (rdm_dateDayName)
#define rdm_dateFromString RDM_T_FUNC (rdm_dateFromString)
#define rdm_dateMonthAbr RDM_T_FUNC (rdm_dateMonthAbr)
#define rdm_dateMonthName RDM_T_FUNC (rdm_dateMonthName)
#define rdm_dateNowAsString RDM_T_FUNC (rdm_dateNowAsString)
#define rdm_dateToString RDM_T_FUNC (rdm_dateToString)
#endif

/** \defgroup date Date API
 * \ingroup datetime
 *
 * The RDM date datatype API.  The functions here are located in \lib{base}
 *
 * \addtogroup date
 * @{
 */

#define RDM_DATE_MAX RDM_UINT32_MAX

#if defined(__cplusplus)
extern "C" {
#endif

/** \brief Get the day abbreviation
 *
 *  Get the three letter day abbreviation in English from a packed date.
 *
 * \return The day abbreviation
 */
RDM_EXPORT const RDM_TCHAR_T *EXTERNAL_FCN rdm_dateDayAbr (
    RDM_PACKED_DATE_T dt /**< [in]  RDM date value */
);

/** \brief Get the day name
 *
 *  Get the day name in English from a packed date.
 *
 * \return The day name
 */
RDM_EXPORT const RDM_TCHAR_T *EXTERNAL_FCN rdm_dateDayName (
    RDM_PACKED_DATE_T dt /**< [in]  RDM date value */
);

/** \brief Get the day of the month
 *
 *  Get the day of month from a packed date.
 *
 * \return The day of month (1 - 31)
 */
RDM_EXPORT uint16_t EXTERNAL_FCN rdm_dateDayOfMonth (
    RDM_PACKED_DATE_T dt /**< [in]  RDM packed date value */
);

/** \brief Get the day of the week
 *
 *  Get the day of week from a packed date for a US locale where
 *  Sunday is the first day of the week.
 *
 * \return The day of month (1 - 7)
 */
RDM_EXPORT uint16_t EXTERNAL_FCN rdm_dateDayOfWeek (
    RDM_PACKED_DATE_T dt /**< [in]  RDM packed date value */
);

/** \brief Get the day of the year
 *
 *  Get the day of year from a packed date where January 1st is the
 *  first day.
 *
 * \return The day of the year (1 - 366)
 */
RDM_EXPORT uint16_t EXTERNAL_FCN rdm_dateDayOfYear (
    RDM_PACKED_DATE_T dt /**< [in]  RDM packed date value */
);

/** \brief Convert from a string to a date
 *
 *  Convert a string to a packed date.
 *
 *  \retval{sOKAY}
 *  \retval{eBASE_INVDATE}
 *  \retval{eINVDATEFMT}
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dateFromString (
    const RDM_TCHAR_T *str, /**< [in] The date as a string that is to be converted */
    RDM_DATE_FORMAT date_fmt, /**< [in] The date format */
    RDM_PACKED_DATE_T *pdv /**< [out] RDM packed date value.  The buffer need to
                              be allocated by the caller. */
);

/** \brief Get the month
 *
 *  Get the month from a packed date.
 *
 *  \return The month (1 - 12)
 */
RDM_EXPORT uint16_t EXTERNAL_FCN rdm_dateMonth (
    RDM_PACKED_DATE_T dt /**< [in]  RDM packed date value */
);

/** \brief Get the month abbreviation
 *
 * Get the three letter month abbreviation in English.
 *
 * \return The month abbreviation
 */
RDM_EXPORT const RDM_TCHAR_T *EXTERNAL_FCN rdm_dateMonthAbr (
    RDM_PACKED_DATE_T dt /**< [in]  RDM date value */
);

/** \brief Get the month name
 *
 * Get the month name in English from a packed date.
 *
 * \return The month name
 */
RDM_EXPORT const RDM_TCHAR_T *EXTERNAL_FCN rdm_dateMonthName (
    RDM_PACKED_DATE_T dt /**< [in]  RDM date value */
);

/** \brief Get the current date as a string
 *
 *  Get the current local date as a string.
 *
 *  \retval{sOKAY}
 *  \retval{eBASE_INVTIMEZONE}
 *
 *  \see rdm_dateToday
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dateNowAsString (
    int16_t time_zone,    /**< [in]  local timezone (in min) */
    RDM_TCHAR_T *datebuf, /**< [out] NOW value in string */
    size_t buflen /**< [in]  size of datebuf in number of ASCII characters it can hold */
);

/** \brief Get the quarter of the year
 *
 *  Get the quarter of the year from a packed date.
 *
 *  \return The quarter of the year (1 - 4)
 */
RDM_EXPORT uint16_t EXTERNAL_FCN rdm_dateQuarter (
    RDM_PACKED_DATE_T dt /**< [in]  RDM packed date value */
);

/** \brief Get the current date
 *
 *  Get the current date based for a time zone.
 *
 *  \retval{sOKAY}
 *  \retval{eBASE_INVTIMEZONE}
 *
 *  \see rdm_dateNowAsString
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dateToday (
    int16_t time_zone,     /**< [in]  local timezone value (in min) */
    RDM_PACKED_DATE_T *pdt /**< [out] current date adjusted with time_zone */
);

/** \brief Convert a date to a string
 *
 *  Convert a packed date to a string.
 *
 *  \retval{sOKAY}
 *  \retval{eBASE_INVDATE}
 *  \retval{eINVDATESEP}
 *  \retval{sTRUNCATE}
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_dateToString (
    RDM_PACKED_DATE_T dateVal, /**< [in]  RDM date value */
    RDM_DATE_FORMAT date_fmt,  /**< [in]  RDM date format */
    RDM_TCHAR_T date_sep,      /**< [in]  date separator */
    RDM_TCHAR_T *buf,          /**< [out] resulting date string */
    size_t bufSize, /**< [in]  buffer size in number of ASCII characters it can hold */
    size_t *puSize /**< [out] resulting size in number of characters */
);

/** \brief Get the week of the year
 *
 *  Get the week of the year in US locale.
 *
 *  \return The week of the year (1 - 53)
 */
RDM_EXPORT uint16_t EXTERNAL_FCN rdm_dateWeek (
    RDM_PACKED_DATE_T dt /**< [in]  RDM packed date value */
);

/** \brief Get the year
 *
 *  Get the year from a packed date.
 *
 *  \return The year
 */
RDM_EXPORT int32_t EXTERNAL_FCN rdm_dateYear (
    RDM_PACKED_DATE_T dt /**< [in]  RDM packed date value */
);

/** \brief Get the date of the epoch
 *
 *  Get the date of the epoch in packed date format.
 *
 *  \return The date of the epoch
 */
RDM_EXPORT RDM_PACKED_DATE_T EXTERNAL_FCN rdm_dateZero (void);

#if defined(__cplusplus)
}
#endif

/** @}*/
